# -*- coding: utf-8 -*-
"""
Created on Fri Dec 25 11:58:20 2020

@author: VODENA06
"""

# Importing Black Fox service libraries

from blackfox import BlackFox
from blackfox import AnnOptimizationConfig, RandomForestOptimizationConfig, XGBoostOptimizationConfig
from blackfox import AnnOptimizationEngineConfig, OptimizationEngineConfig
from blackfox import LogWriter, CsvLogWriter
from blackfox_extras import prepare_input_data, scale_output_data
from blackfox import InputConfig, Encoding
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
import matplotlib as mpl
mpl.style.use('ggplot')
import warnings
warnings.filterwarnings("ignore")
from sklearn.model_selection import train_test_split
from sklearn.metrics import confusion_matrix
from sklearn.preprocessing import StandardScaler, MinMaxScaler
import h5py
import time
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
import matplotlib as mpl
mpl.style.use('ggplot')
import warnings
warnings.filterwarnings("ignore")
from sklearn.model_selection import train_test_split
from sklearn.metrics import confusion_matrix
from sklearn.preprocessing import StandardScaler, MinMaxScaler
import h5py
import time

# blackfox_url = 'http://147.91.204.14:32702' # BF_URL
# blackfox_url = 'http://147.91.204.85:32723'# ~VODAFONE_URL
blackfox_url = 'http://147.91.204.14:32706' # BF FOR MAPE verziju blackfox 3.2.0
# blackfox_url = 'http://localhost:5000'
bf = BlackFox(blackfox_url)


dataframe = pd.read_csv('F:\\.spyder-py3\\projekti\\Vodena_Primeri\\ImplementingEncodingInBf\\data\\data_set.csv')


y = dataframe.iloc[:, -1:].values
X = dataframe.iloc[:, :-1].values

X_train, X_test, y_train, y_test = train_test_split(X, y, test_size = 0.2, random_state = 0)

# X_train, X_val, y_train, y_val = train_test_split(X_train, y_train, test_size = 0.2, random_state = 0)


start = time.time()

ec = AnnOptimizationEngineConfig(
    population_size = 50,
    max_num_of_generations = 10
)

c = AnnOptimizationConfig(
    problem_type = "BinaryClassification",
    binary_optimization_metric = "ROC_AUC",
    engine_config = ec,
    inputs=[
        InputConfig(encoding='OneHot', is_optional=True),
        InputConfig(encoding='Dummy', is_optional=True),
        InputConfig(encoding='Target', is_optional=True),
        InputConfig(encoding='Effect', is_optional=True)
    ]
    )

# Use CTRL + C to stop optimization
(ann_io, ann_info, ann_metadata) = bf.optimize_ann(
    input_set = X_train,
    output_set = y_train,
    config = c,
    model_path = 'test_model_encoding_version_bf_feature_selection_f.h5',
    delete_on_finish = False
)

end = time.time()
time_BF_ann = int(end-start)

print('\nann info:')
print(ann_info)

print('\nann metadata:')
print(ann_metadata)



# Get model metadata
model_name = 'F:\\.spyder-py3\\projekti\\Vodena_Primeri\\ImplementingEncodingInBf\\data\\test_model_encoding_version_bf_feature_selection_f.h5'
metadata = bf.get_ann_metadata(model_name)
print(metadata)

# Prepare input data for prediction
X_test_prepared_with_BF = prepare_input_data(X_test, metadata)





# Load model
from tensorflow.keras.models import load_model
model_BF_ann = load_model(model_name)

# Prediction and scale predicted data
y_pred_BF_ann = model_BF_ann.predict(X_test_prepared_with_BF)
y_pred_BF_ann = scale_output_data(y_pred_BF_ann, metadata)
print(y_test)








from sklearn.ensemble import RandomForestClassifier, RandomForestRegressor
from sklearn.multioutput import MultiOutputRegressor
import tensorflow
import joblib
import xgboost
import os

def load_model(path):
    try:
        model = tensorflow.keras.models.load_model(path)
        return model
    except:
        try:
            model = joblib.load(path)
            return model
        except:
            try:
                model = xgboost.XGBClassifier()
                model.load_model(path)
                return model
            except:
                try:
                    model = xgboost.XGBRegressor()
                    model.load_model(path)
                    return model
                except:
                    try:
                        model = MultiOutputRegressor()
                        model.load_model(path)
                        return model
                    except:
                        raise Exception('An error occurred during model loading.')


import numpy as np

def pack_input_data_for_rnn(input_data, metadata):
    """
        Packing input data for recurrent neural network

        Parameters
        ----------
        data : numpy.array
            Data as numpy array
        metadata : dict
            Model metadata

        Returns
        -------
        numpy.array
            Packed data
    """
    rnn_input_count = metadata['recurrent_input_count']
    rnn_output_count = metadata['recurrent_output_count']

    x_data = []
    rows_count = input_data.shape[0] - rnn_output_count
    for i in range(rnn_input_count, rows_count):
        x_data.append(input_data[i - rnn_input_count:i])

    new_shape = (rows_count - rnn_input_count, rnn_input_count, input_data.shape[1])
    x_data = np.reshape(x_data, new_shape)
    return x_data

def pack_output_data_for_rnn(output_data, metadata):
    """
        Packing output data for recurrent neural network

        Parameters
        ----------
        data : numpy.array
            Data as numpy array
        metadata : dict
            Model metadata

        Returns
        -------
        numpy.array
            Packed data
    """
    rnn_input_count = metadata['recurrent_input_count']
    rnn_output_count = metadata['recurrent_output_count']

    y_data = []
    rows_count = output_data.shape[0] - rnn_output_count
    for i in range(rnn_input_count, rows_count):
        y_data.append(output_data[i:i + rnn_output_count].flatten())

    y_data = np.array(y_data)
    return y_data


from blackfox import LogWriter

class ChartLogWriter(LogWriter):

    def __init__(self):
        pass

    def write_status(self, id, status):
        pass

    def write_string(self, msg):
        pass


import pandas as pd
import numpy as np
from sklearn.model_selection import train_test_split

dataframe = pd.read_csv('C:\\Users\\VODENA06\\Desktop\\Jupyter\\BF\\BF_TESTIRANJE\\5_0_0\\Churn_Modelling.csv')

print(dataframe.columns)

# df_X = dataframe.iloc[:, 3:13]
y = dataframe.iloc[:, 13:14].values
print(y[:5,:])
y = np.array([3 if elem==0 else 5 for elem in y]).reshape(len(y), 1)
print(y[:5,:])

#df_X = pd.concat(
#    [
#     pd.get_dummies(df_X['Geography'], drop_first = True),
#     pd.get_dummies(df_X['Gender'], drop_first = True),
#     df_X.drop(['Geography', 'Gender'], axis=1),
#     ], axis=1)
#X = df_X.iloc[:, :].values
X = dataframe.iloc[:, 3:13].values
print(X[:3,:])

from sklearn.preprocessing import LabelEncoder
le = LabelEncoder()
X = np.concatenate(
    [
    X[:, 0:1],
    le.fit_transform(X[:, 1:2]).reshape(len(X),1),
    le.fit_transform(X[:, 2:3]).reshape(len(X),1),
    X[:, 3:]
    ], axis=1)
print(X[:3,:])

# Split the entire data set into the training data and test set

X_train, X_test, y_train, y_test = train_test_split(X, y, test_size = 0.2, random_state = 0)
print('shape 0 :{}      and shape 1 :{}'.format(X_train.shape[0], X_train.shape[1]))
print('shape 0 :{}      and shape 1 :{}'.format(X_test.shape[0], X_test.shape[1]))
print('shape 0 :{}      and shape 1 :{}'.format(y_train.shape[0], y_train.shape[1]))
print('shape 0 :{}      and shape 1 :{}\n'.format(y_test.shape[0], y_test.shape[1]))

# Split the training dada into the training set and validation set

X_train, X_val, y_train, y_val = train_test_split(X_train, y_train, test_size = 0.2, random_state = 0)
print('shape 0 :{}      and shape 1 :{}'.format(X_train.shape[0], X_train.shape[1]))
print('shape 0 :{}      and shape 1 :{}'.format(X_val.shape[0], X_val.shape[1]))
print('shape 0 :{}      and shape 1 :{}'.format(y_train.shape[0], y_train.shape[1]))
print('shape 0 :{}      and shape 1 :{}\n'.format(y_val.shape[0], y_val.shape[1]))




# Get model metadata
metadata = {'__version': 8, 'scaler_name': 'MinMaxScaler', 'scaler_config': {'input': {'feature_range': [-1, 1], 'fit': [[350.0, 0.0, 0.0, 0.0, 0, 18.0, 0.0, 0.0, 1.0, 0.0, 0.0, 11.58], [850.0, 1.0, 1.0, 1.0, 1, 85.0, 10.0, 238387.56, 4.0, 1.0, 1.0, 199992.48]], 'inverse_transform': False}, 'output': {'feature_range': [0, 1], 'fit': [[0.0], [1.0]], 'inverse_transform': True}}, 'is_scaler_integrated': False, 'has_rolling': False, 'binary_metric': 'ROC_AUC', 'metric_threshold': None, 'input_encodings': [{'type': 'None'}, {'type': 'OneHot', 'categories': ['0', '1', '2']}, {'type': 'OrderInteger', 'mapping': {'1': 0, '0': 1}}, {'type': 'None'}, {'type': 'None'}, {'type': 'None'}, {'type': 'None'}, {'type': 'None'}, {'type': 'None'}, {'type': 'None'}], 'output_encoding': [{'type': 'Dummy', 'categories': ['3', '5']}]}

# Prepare input data for prediction
from data_extras import prepare_input_data
X_test_prepared_with_BF = prepare_input_data(X_test, metadata)

y_pred = [0.2,0.3,0.6,0.9,0,1,0.9,0,0,0,0,0,0]
from encode_variable import VariableEncoding
y_decoded = VariableEncoding.decode(y_pred, metadata)

print('Done.')


# coding: utf-8

"""
    BlackFoxExtras

    BlackFoxExtras

"""


from setuptools import setup, find_packages  # noqa: H301

NAME = "blackfox_extras"
VERSION = "0.2.2"

REQUIRES = ["blackfox >= 0.0.8", "numpy >= 1.16.2", "scikit-learn >= 0.21.2", "pandas >= 1.1.4", "category_encoders >= 2.2.2"]

setup(
    name=NAME,
    version=VERSION,
    description="BlackFox Extras",
    author="Tomislav Mrdja",
    author_email="",
    url="https://github.com/vodena/BlackFoxPythonExtras",
    keywords=["BlackFox"],
    install_requires=REQUIRES,
    packages=find_packages(exclude=["test.*", "test"]),
    include_package_data=True,
    long_description="""\
        BlackFox Extras
    """
)





def remove_not_used_inputs(data, metadata):
    """
        Remove insignificant columns from data, if model has feature selection

        Parameters
        ----------
        data : numpy.array
            Data as numpy array
        metadata : dict
            Model metadata

        Returns
        -------
        numpy.array
            New data
        """
    if 'is_input_used' in metadata:
        tuples = [[x, i] for i, x in enumerate(metadata['is_input_used'])]
        used = filter(lambda tu: tu[0], tuples)
        index = list(map(lambda t: t[1], used))
        return data[:, index]
    else:
        return data
